@@ -5,6 +5,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"log/slog"
 
 	"github.com/ClickHouse/clickhouse-go/v2/lib/proto"
 )
@@ -76,7 +77,7 @@ func (c *connect) firstBlockImpl(ctx context.Context, on *onProcess) (*proto.Blo
 			return c.readData(ctx, packet, true)
 
 		case proto.ServerEndOfStream:
-			c.debugf("[end of stream]")
+			c.logger.Debug("end of stream received")
 			return nil, io.EOF
 
 		default:
@@ -147,7 +148,7 @@ func (c *connect) processImpl(ctx context.Context, on *onProcess) error {
 
 		switch packet {
 		case proto.ServerEndOfStream:
-			c.debugf("[end of stream]")
+			c.logger.Debug("end of stream received")
 			return nil
 		}
 
@@ -177,14 +178,17 @@ func (c *connect) handle(ctx context.Context, packet byte, on *onProcess) error
 		if err := info.Decode(c.reader, c.revision); err != nil {
 			return err
 		}
-		c.debugf("[profile info] %s", &info)
+		c.logger.Debug("profile info received",
+			slog.Uint64("rows", info.Rows),
+			slog.Uint64("blocks", info.Blocks),
+			slog.Uint64("bytes", info.Bytes))
 		on.profileInfo(&info)
 	case proto.ServerTableColumns:
 		var info proto.TableColumns
 		if err := info.Decode(c.reader, c.revision); err != nil {
 			return err
 		}
-		c.debugf("[table columns]")
+		c.logger.Debug("table columns received")
 	case proto.ServerProfileEvents:
 		scanEvents := on.profileEvents != nil
 		events, err := c.profileEvents(ctx, scanEvents)
@@ -205,7 +209,7 @@ func (c *connect) handle(ctx context.Context, packet byte, on *onProcess) error
 		if err != nil {
 			return err
 		}
-		c.debugf("[progress] %s", progress)
+		// Progress is already logged in c.progress()
 		on.progress(progress)
 	default:
 		return &OpError{
@@ -217,7 +221,7 @@ func (c *connect) handle(ctx context.Context, packet byte, on *onProcess) error
 }
 
 func (c *connect) cancel() error {
-	c.debugf("[cancel]")
+	c.logger.Debug("cancelling query")
 	c.buffer.PutUVarInt(proto.ClientCancel)
 	wErr := c.flush()
 	// don't reuse a cancelled query as we don't drain the connection
