@@ -1,13 +1,17 @@
 package clickhouse
 
 import (
+	"log/slog"
+
 	"github.com/ClickHouse/clickhouse-go/v2/lib/proto"
 )
 
 // Connection::sendQuery
 // https://github.com/ClickHouse/ClickHouse/blob/master/src/Client/Connection.cpp
 func (c *connect) sendQuery(body string, o *QueryOptions) error {
-	c.debugf("[send query] compression=%q %s", c.compression, body)
+	c.logger.Debug("sending query",
+		slog.String("compression", c.compression.String()),
+		slog.String("query", body))
 	c.buffer.PutByte(proto.ClientQuery)
 	q := proto.Query{
 		ClientTCPProtocolVersion: ClientTCPProtocolVersion,
